#include <iostream>
#include <stdexcept>
#include <windows.h>

#ifndef MOD_NOREPEAT
#define MOD_NOREPEAT 0x4000
#endif

DISPLAY_DEVICE getDisplayDevice(DWORD displayId) {
	// Setting cb is important, otherwise the query will succeed but won't have any data
	DISPLAY_DEVICE device{};
	device.cb = sizeof(DISPLAY_DEVICE);

	if(!EnumDisplayDevices(NULL, displayId, &device, 0)) {
		throw std::invalid_argument("invalid deviceId");
	}

	return device;
}

DEVMODE getDisplaySettings(DWORD displayId) {
	DEVMODE deviceMode{};
	deviceMode.dmSize = sizeof(DEVMODE);

	if(!EnumDisplaySettings(getDisplayDevice(displayId).DeviceName, ENUM_CURRENT_SETTINGS, &deviceMode)) {
		throw std::invalid_argument("failed to retrieve display settings");
	}
	return deviceMode;
}

void setDisplayFrequency(DWORD displayId, DWORD frequency) {
	DEVMODE deviceMode = getDisplaySettings(displayId);
	deviceMode.dmDisplayFrequency = frequency;
	if(ChangeDisplaySettings(&deviceMode, 0) != DISP_CHANGE_SUCCESSFUL) {
		throw std::runtime_error("failed to change display frequency");
	}
}

void registerShortcut(int shortcutId, UINT key) {
	// registers a shorctut ALT + key
	// the shortcutId is used to distinguish between multiple keys
	if(!RegisterHotKey(NULL, shortcutId, MOD_ALT | MOD_NOREPEAT, key)) {
		throw std::runtime_error("failed to register hotkey");
	}
}

void runEventLoop(DWORD displayId) {
	MSG msg{};

	while(GetMessage(&msg, NULL, 0, 0) > 0) {
		switch(msg.message) {
		case WM_HOTKEY:
			// msg.wParam holds the shortcutid
			std::cout << "switching display frequency to " << msg.wParam << "\n";
			setDisplayFrequency(displayId, msg.wParam);
			break;
		}
	}
}

int main() {
	try {
		// Make shortcut ALT + B change the frequency to 23
		registerShortcut(23, 0x42);

		// Make shortcut ALT + V change the frequency to 60
		registerShortcut(60, 0x56);

		// Make keypresses switch the frequency of the display at index 1
		runEventLoop(1);
	} catch(std::exception const& e) {
		std::cout << "Error during execution: " << e.what() << "\n";
	}
}
